<?php
// $Id: memetracker.inc,v 1.2 2008/07/08 17:10:37 kylemathews Exp $

/**
 * @file
 * The theme system, which controls the output of Drupal.
 *
 * The theme system allows for nearly all output of the Drupal system to be
 * customized by user themes.
 */


include_once './'. drupal_get_path('module', 'machinelearningapi') .'/machine_learning_api.inc';
include_once 'meme.inc';
include_once 'content.inc';
include_once 'content_source.inc';


/**
 * class Memetracker
 */
class Memetracker {

  /** Aggregations: */

  /** Compositions: */
  var $m_;

   /*** Attributes: ***/
  
  // Memetracker ID
  private $mid;
  private $content_sources = array();
  private $content_array = array();
  private $ml_api_obj;
  private $ml_api_name;
  private $last_cron;
  private $distance_threshold;
  public $num_memes;
  
  /*
   * Constructor // load time between redoing memes + redoing interest / discussion (discussions redone every cron by default? to encourage discussion?)
   // add fields to memetracker table
   // add code to 
   */
  function __construct($mid) {
    $result = db_result(db_query("SELECT mid FROM {memetracker} 
    WHERE mid = %d", $mid));
    // Only build memetracker if there's an already existing memetracker 
    // in the database.
    if ($result == $mid) {
      $this->mid = $mid;
      $tw = time() - 86400 * 2; 

      // Load info about memetracker from the database.
      $results = db_fetch_array(db_query("
      SELECT ml_api, last_cron, distance_threshold, num_memes, num_days, name
      FROM {memetracker} 
      WHERE mid = %d", $mid));
      $cs_names = db_query("SELECT cs_name from {memetracker_content_source}
      WHERE mid = %d", $mid);
      
      $cutoff_point = time() - (86400 * $results['num_days']);
#      dpm('in memetracker.inc: '. $cutoff_point);
      $content = db_query("SELECT cid, content_type FROM {memetracker_content}
        WHERE mid = %d and timestamp > %d", $mid, $cutoff_point);

      // Load names of content source objects
      $content_sources = array();
      while ($data = db_fetch_array($cs_names)) {
        $content_sources[] = $data['cs_name'];
      }
      
      // Load array of content objects -- when have sliding window for memetracker
      // this will only load content objects within that window
      $content_array = array();
      while ($data = db_fetch_array($content)) {
        $content_class_string = $data['content_type'];
        $content_array[] = new $content_class_string($this->get_mid(), $data['cid'], Null);
      }
      
#      dpm('count: '. count($content_array));
      
      // Set values of variables pulled from the database
      $this->ml_api_name = $results['ml_api'];
      $this->last_cron = $results['last_cron'];
      $this->distance_threshold = $results['distance_threshold'];
      $this->num_memes = $results['num_memes'];
      $this->num_days = $results['num_days'];
      $this->name = $results['name'];
      $this->set_content_sources($content_sources);
      $this->set_content_array($content_array);
    }
  }
  
  /*
   * Set content_sources
   */
  public function set_content_sources($content_sources) {
    $this->content_sources = $content_sources;
  }
  
  /*
   * Get content_sources
   */
  public function get_content_sources() {
    return $this->content_sources;
  }
  /*
   * Set content_array 
   */
  public function set_content_array($content_array) {
    $this->content_array = $content_array;
  }
  
  /*
   * Get Memetracker ID
   */
  public function get_mid() {
    return $this->mid;
  }
  
  /*
   * Get distance_threshold
   */
  function get_distance_threshold() {
    return $this->distance_threshold;
  }
  
  /*
   * Singleton pattern to return instance of machine learner
   */
  private function get_ml_instance() {
    if (is_null($this->ml_api_obj)) {
      $this->ml_api_obj = new $this->ml_api_name();
      return $this->ml_api_obj;
    }
    else {
      return $this->ml_api_obj;
    }
  }
  
  /*
   * Use for testing only
   */
  public function test_get_ml_instance() {
    return $this->get_ml_instance();
    
  }
  
  public function load_content_source($class_name) {  
    if (is_string($class_name)) {
      return new $class_name($this->mid);
    }
  }
  /**
   *
   * @return 
   * @access public
   */
  public function check_new_content( ) {
    // loop through $content_sources calling get_content on each
    foreach   ($this->content_sources as $content_source_string) {
      $cs_obj = $this->load_content_source($content_source_string);
      $new_content = $cs_obj->get_content_array();
      
      // If the array is null, create an empty array so the array_merge function
      // doesn't complain.
      if (is_null($new_content)) {
        $new_content = array();
      }

      // add new content objects to existing content objects
      $this->content_array = array_merge($this->content_array, $new_content);
    }
  } // end of member function check_new_content
  
  /*
   * Check for new feed items
   */ 
  function check_new_feed_items($feeds) {
    $cs_obj = $this->load_content_source('content_source_drupal_feedapi');
    $cs_obj->data = $feeds;
    $cs_obj->last_cron = 0;
    
    $new_content = $cs_obj->get_content_array();

    // If the array is null, create an empty array so the array_merge function
    // doesn't complain.
    if (is_null($new_content)) {
      $new_content = array();
    }

    // add new content objects to existing content objects
    $this->content_array = array_merge($this->content_array, $new_content);

  }

  /**
   *
   * @param content content_array 
   * @return meme
   * @access public
   */
  public function get_memes($reset_cache = False) {
    if ($reset_cache){
      $ml_api = $this->get_ml_instance();
      
      // Set interestingness score on content objects 
      // if they don't have one already.
      $ml_api->calculate_content_interestingness($this->content_array);
      
      // Get memes
      $memes = $ml_api->get_memes($this->content_array, 
      $this->distance_threshold, $this->num_days);

      cache_set('memetracker_memes'. $this->mid, $memes);
      return;
    }
    else {
      $memes = cache_get('memetracker_memes'. $this->mid)->data;
    }      
    
    if ($memes) {
      array_splice($memes, $this->num_memes);
    }
    
    return $memes;
    
  } // end of member function get_memes
  
  public function train_on_click_data() {
    $ml_api = $this->get_ml_instance(); // get_machine_learner
    $ml_api->train_on_click_data($this->content_array, $this->last_cron);
  }
  
  /*
   * Saves time of cron run to memetracker table
   */
  function save_cron_time() {
    $this->last_cron = time();
    db_query("UPDATE {memetracker} SET last_cron = %d WHERE mid = %d", 
    time(), $this->mid);
  }
}
